<?php

class OxidFacade
{
    /**
     * @return string
     */
    public static function getVersion()
    {
        if (class_exists('oxRegistry', false)) {
            return oxRegistry::getConfig()->getVersion();
        } else {
            return oxConfig::getInstance()->getVersion();
        }
    }

    /**
     * @param string $varName
     * @return mixed
     */
    public static function getShopConfVar($varName)
    {
        if (self::needRegistry()) {
            return oxRegistry::getConfig()->getShopConfVar($varName);
        } else {
            // TODO: Clarify, if getParameter() or getConfigParam() is correct
            return oxConfig::getInstance()->getConfigParam($varName);
        }
    }

    /**
     * @return oxbasket
     */
    public static function getBasket()
    {
        if (self::needRegistry()) {
            return oxRegistry::getSession()->getBasket();
        } else {
            return oxSession::getInstance()->getBasket();
        }
    }

    /**
     * @param double $number
     * @return string
     */
    public static function formatCurrency($number)
    {
        if (self::needRegistry()) {
            return oxRegistry::getLang()->formatCurrency($number);
        } else {
            return oxLang::getInstance()->formatCurrency($number);
        }
    }

    /**
     * @param double $number
     * @return float
     */
    public static function fRound($number)
    {
        if (self::needRegistry()) {
            return (double)oxRegistry::getUtils()->fRound($number);
        } else {
            return (double)oxUtils::getInstance()->fRound($number);
        }
    }

    /**
     * @param string $text
     * @return string
     */
    public static function translateString($text)
    {
        if (self::needRegistry()) {
            return oxRegistry::getLang()->translateString($text);
        } else {
            return oxLang::getInstance()->translateString($text);
        }
    }

    /**
     * @param string $parameter
     * @return mixed
     */
    public static function getRequestParameter($parameter)
    {
        if (self::needRegistry()) {
            return oxRegistry::getConfig()->getRequestParameter($parameter);
        } else {
            return oxConfig::getParameter($parameter);
        }
    }

    /**
     * @return boolean
     */
    protected static function needRegistry()
    {
        return (version_compare(self::getVersion(), '4.7.0', '>='));
    }
    
    /**
     * @return string
     */
    public static function getShopUrl()
    {
        if (self::needRegistry()) {
            return oxRegistry::getConfig()->getShopUrl();
        } else {
            return oxConfig::getInstance()->getShopUrl();
        }
    }
}
